#include <stdio.h>
#include <omp.h>
#include <math.h>

void do_work_parallel(void)
{
	int i;
	int n = 1000000;
	double PI25DT = 3.141592653589793238462643;
	double pi, h, area, x;

	h = 1.0 / (double) n;
	area = 0.0;
	#pragma omp parallel for private(x) reduction(+:area)
	for (i = 1; i <= n; i++)
	{
		x = h * ((double)i - 0.5);
		area += (4.0 / (1.0 + x*x));
	}
	pi = h * area;
	printf("pi (by using #pragma omp parallel for) is approximately %.16f, Error is %.16f\n",pi,fabs(pi - PI25DT));
}

void do_work_sections(void){
	int i;
	int n = 1000000;
	double PI25DT = 3.141592653589793238462643;
	double pi, h, area, x;

	h = 1.0 / (double) n;
	area = 0.0;
	#pragma omp parallel sections private(i,x) reduction(+:area)
	{
		#pragma omp section
		for (i = 1; i < n/2; i++)
		{
			x = h * ((double)i - 0.5);
			area += (4.0 / (1.0 + x*x));
		}

		#pragma omp section
		for (i = n/2; i <= n; i++)
		{
			x = h * ((double)i - 0.5);
			area += (4.0 / (1.0 + x*x));
		}
	}
	pi = h * area;
	printf("pi (by using #pragma omp parallel sections) is approximately %.16f, Error is %.16f\n",pi,fabs(pi - PI25DT));
}

int main(int argc, char **argv)
{

	do_work_parallel();

	#pragma omp parallel
	{
		#pragma omp critical (foo)
		printf ("critical foo\n");
		#pragma omp critical (bar)
		printf ("critical bar\n");
		#pragma omp critical (foo)
		printf ("critical foo (again)\n");
		#pragma omp critical
		printf ("critical unnamed\n");
	}

	do_work_sections();
}
